'use strict';

var assign = require('object-assign');

module.exports = {
    defaults: {
        //CONTAINER STYLE
        defaultContainerStyle: {
            // overflow: 'auto',
            position: 'relative'
        },
        //overriden by
        containerStyle: {
            transform: 'translate3d(0px, 0px, 0px)'
        },

        //ORIENTATION STYLE
        defaultHorizontalContainerStyle: {
            width: '100%',
            height: '50%'
        },
        defaultVerticalContainerStyle: {
            // height: '100%',
            width: '50%',
            float: 'left'
        },
        //overriden by
        horizontalContainerStyle: {},
        verticalContainerStyle: {},

        //INDEX STYLE
        defaultFirstContainerStyle: {},
        defaultSecondContainerStyle: {},
        //overriden by
        firstContainerStyle: {},
        secondContainerStyle: {},

        //ORIENTATION-INDEX STYLE
        defaultHorizontalFirstContainerStyle: {},
        defaultHorizontalSecondContainerStyle: {},
        defaultVerticalFirstContainerStyle: {},
        defaultVerticalSecondContainerStyle: {},
        //overriden by
        horizontalFirstContainerStyle: {},
        horizontalSecondContainerStyle: {},
        verticalFirstContainerStyle: {},
        verticalSecondContainerStyle: {}
    },

    prepare: function prepare(props) {
        var H = props.orientation == 'horizontal';

        ////////// DEFAULTS ////////////
        //CONTAINER STYLE
        var defaultContainerStyle = props.defaultContainerStyle;

        //ORIENTATION STYLE
        var defaultOrientationStyle = H ? props.defaultHorizontalContainerStyle : props.defaultVerticalContainerStyle;

        //INDEX STYLE
        var defaultFirstStyle = props.defaultFirstContainerStyle;
        var defaultSecondStyle = props.defaultSecondContainerStyle;

        //ORIENTATION-INDEX STYLE
        var defaultOrientationFirstStyle = H ? props.defaultHorizontalFirstContainerStyle : props.defaultVerticalFirstContainerStyle;
        var defaultOrientationSecondStyle = H ? props.defaultHorizontalSecondContainerStyle : props.defaultVerticalSecondContainerStyle;

        //container
        var containerStyle = assign({}, defaultContainerStyle, props.containerStyle);
        //orientation
        var orientationStyle = assign({}, defaultOrientationStyle, H ? props.horizontalContainerStyle : props.verticalContainerStyle);
        //index
        var firstContainerStyle = assign({}, defaultFirstStyle, props.firstContainerStyle);
        var secondContainerStyle = assign({}, defaultSecondStyle, props.secondContainerStyle);
        //orientation-index
        var orientationFirstStyle = assign({}, defaultOrientationFirstStyle, H ? props.horizontalFirstContainerStyle : props.verticalFirstContainerStyle);
        var orientationSecondStyle = assign({}, defaultOrientationSecondStyle, H ? props.horizontalSecondContainerStyle : props.verticalSecondContainerStyle);

        // ------ container style
        //       |
        //       |
        //      \`/
        // ------- orientation style
        //       |
        //       |
        //      \`/
        // ------- index style
        //       |
        //       |
        //      \`/
        // ------- orientation-index style
        return {
            first: assign({}, containerStyle, orientationStyle, firstContainerStyle, orientationFirstStyle),
            second: assign({}, containerStyle, orientationStyle, secondContainerStyle, orientationSecondStyle)
        };
    }
};