import React, { PropTypes } from 'react'
import { findDOMNode } from 'react-dom'

import Component from 'react-class'

import assign from 'object-assign'
import normalize from 'react-style-normalizer'

import Splitter from './Splitter'
import STYLES from './styles'

import join from './join'

import bemFactory from './bemFactory'

const PERCENTAGE_RE = /%/

import isNumber from './isNumber'
import calc from './calc'

const CLASS_NAME = 'react-split-container'
const bem = bemFactory(CLASS_NAME)
const m = (name) => bem(null, name)

const isPercentage = (n) => {
  if (isNumber(n)){
      return false
  }

  return typeof n == 'string' && isNumber(n.replace(PERCENTAGE_RE, ''))
}

const getSizeName = (props) => {
  var H = props.orientation == 'horizontal'
  return H? 'height': 'width'
}

const getPositionNames = (props) => {
  var H = props.orientation == 'horizontal'
  return H? ['top', 'bottom']: ['left','right']
}

const getClientSizeName = (props) => {
  var sizeName = getSizeName(props)
  return 'client' + sizeName[0].toUpperCase() + sizeName.substring(1)
}

const getFixedIndex = (props) => {
    if (typeof props.fixedIndex == 'string'){
        props.fixedIndex *= 1
    }

    return props.fixedIndex === 0?
            0:
            props.fixedIndex === 1?
                1:
                -1
}
const emptyFn = () => {}

export default class SplitContainer extends Component {

  constructor(props){
    super(props)

    this.state = {
      dragging: false
    }
  }

  isMounted(){
    return !!findDOMNode(this)
  }

  prepareStyle(props, state) {

    const H = props.orientation == 'horizontal'

    const defaultOrientationStyle = H?
      props.defaultHorizontalStyle:
      props.defaultVerticalStyle

    const style = assign({}, props.defaultStyle, props.style)

    if (state.dragging){
      assign(style, {
        cursor: H? 'ns-resize':'ew-resize'
      })
    }

    return assign(style, defaultOrientationStyle)
  }

  adjustMinSizes(splitterSize, props, state, styles) {
      var H = props.orientation == 'horizontal'
      var minSizeName = H? 'minHeight': 'minWidth'
      var maxSizeName = H? 'maxHeight': 'maxWidth'

      var totalMinSize = 0

      var collapsedInfo = this.getCollapsedInfo(props, state)

      if (props.firstMinSize){
          totalMinSize += props.firstMinSize
          styles.first[minSizeName] = props.firstMinSize
          styles.second[maxSizeName] = calc('100%', -props.firstMinSize, -splitterSize)
      }

      if (props.secondMinSize){
          totalMinSize += props.secondMinSize
          styles.second[minSizeName] = props.secondMinSize
          styles.first[maxSizeName]  = calc('100%', -props.secondMinSize, -splitterSize)
      }

      if (collapsedInfo.first){
          delete styles.second[maxSizeName]
      }

      if (collapsedInfo.second){
          delete styles.first[maxSizeName]
      }

      if (totalMinSize){
          props.style[minSizeName] = totalMinSize
      }
  }

  prepareSplitterStyle(props, state) {
      var H = props.orientation == 'horizontal'
      var sizeName = getSizeName(props)

      var defaultOrientationSplitterStyle = H? props.defaultHorizontalSplitterStyle: props.defaultVerticalSplitterStyle
      var splitterStyle = assign({}, props.defaultSplitterStyle, defaultOrientationSplitterStyle, props.splitterStyle)

      if (props.splitterSize){
          splitterStyle[sizeName] = props.splitterSize
      }

      if (typeof splitterStyle[sizeName] === 'undefined'){
          splitterStyle[sizeName] = props.defaultSplitterSize
      }

      if (state.dragging){
          // assign(splitterStyle, state.dragSplitterStyle)
      }

      var collapsedInfo = this.getCollapsedInfo(props, state)

      if (collapsedInfo.first || collapsedInfo.second || props.locked){
          splitterStyle.cursor = 'auto'
      }

      return normalize(splitterStyle)
  }

  getCollapsedInfo(props, state){

      var first = typeof state.firstCollapsed != 'undefined'?
                      state.firstCollapsed:
                      props.firstCollapsed

      var second = typeof state.secondCollapsed != 'undefined'?
                      state.secondCollapsed:
                      props.secondCollapsed

      return {
          first : first,
          second: second
      }
  }

  prepareChildrenStyles(props, state, splitterSize){
      var sizeName     = getSizeName(props)
      var styles       = STYLES.prepare(props)
      var halfSplitter = splitterSize / 2

      var offset       = state.dragOffset || 0
      var splitAt      = props.splitAt
      var FIRST
      var SECOND

      var collapsedInfo = this.getCollapsedInfo(props, state)

      if (collapsedInfo.first || collapsedInfo.second){
          if (collapsedInfo.first){
              SECOND = calc('100%', -splitterSize)
              styles.first.display = 'none'
          } else {
              FIRST = calc('100%', -splitterSize)
              styles.second.display = 'none'
          }
      } else {
          if (isNumber(splitAt)){
              FIRST  = calc(splitAt, offset)
              SECOND = calc('100%', -splitAt, -splitterSize, -offset)
          } else {
              FIRST  = calc(splitAt, -halfSplitter, offset)
              SECOND = calc('100%', '-', splitAt, -halfSplitter, -offset)
          }
      }

      styles.first[sizeName] = FIRST
      styles.second[sizeName] = SECOND

      return styles
  }

  prepareProps(thisProps){
    const props = this.p = assign({}, thisProps)

    const H = props.horizontal = props.orientation == 'horizontal'

    props.className = this.prepareClassName(props)

    return props
  }

  prepareClassName(props){
    return join(
      CLASS_NAME,
      m(props.orientation),
      props.theme && m(`theme-${props.theme}`)
    )
  }

  render() {
      const props = this.p = this.prepareProps(this.props)

      const H = props.horizontal
      const sizeName = H? 'height': 'width'

      this.asPercentage = isPercentage(props.splitAt)

      props.children = props.children || []

      var splitAtInState = typeof this.state.splitAt != 'undefined'

      if (splitAtInState){
          props.splitAt = this.state.splitAt
      }

      props.splitterStyle = this.prepareSplitterStyle(props, this.state)
      props.style         = this.prepareStyle(props, this.state)

      var splitterSize = props.splitterSize = parseFloat(props.splitterStyle[sizeName])

      if (this._adjusting){
          this.adjustBefore(props, this.state, splitterSize)
      }

      var childrenStyles = this.prepareChildrenStyles(props, this.state, splitterSize)

      if (this._adjusting){
          this.adjustAfter(props, this.state, childrenStyles)
      }

      this.adjustMinSizes(splitterSize, props, this.state, childrenStyles)

      if (this.shouldAdjust(props)){

          if (!this._adjusting){
              setTimeout(function(){

                  this._adjusting = true

                  if (!this.isMounted()){
                      return
                  }

                  this.setState({}, function(){
                      this._adjusting = false
                  }.bind(this))
              }.bind(this), 0)
          } else {
              this._adjusting = false
          }
      }

      var collapsedInfo = this.getCollapsedInfo(props, this.state)

      this.state.props = props

      return (
          <div {...props}>
              <div className={join("item first", `${CLASS_NAME}__first`, `${CLASS_NAME}__first--orientation-${props.orientation}`)} style={childrenStyles.first}>
                  {props.children[0]}
              </div>
              <Splitter
                  beforeMin    ={props.firstMinSize}
                  afterMin     ={props.secondMinSize}
                  size         ={splitterSize}
                  arrowSize    ={props.arrowSize}
                  arrowColor   ={props.arrowColor}
                  arrowPadding ={props.arrowPadding}
                  style        ={props.splitterStyle}
                  onDragStart  ={this.handleSplitterDragStart}
                  onDrag       ={this.handleSplitterDrag}
                  onDrop       ={this.handleSplitterDrop}
                  onCollapse   ={this.handleCollapse}
                  onExpand     ={this.handleExpand}
                  orientation  ={props.orientation}
                  locked  ={props.locked}
                  firstCollapsed={collapsedInfo.first}
                  secondCollapsed={collapsedInfo.second}
                  firstCollapsible={props.firstCollapsible}
                  secondCollapsible={props.secondCollapsible}
                  onMouseDown={this.handleSplitterMouseDown}
              />
              <div className={join("item first", `${CLASS_NAME}__second`, `${CLASS_NAME}__second--orientation-${props.orientation}`)} style={childrenStyles.second}>
                  {props.children[1]}
              </div>
          </div>
      )
  }

  getSplitterSize(props) {
      props = props || this.props

      var splitterStyle = props.splitterStyle

      return splitterStyle[getSizeName(props)] || 10
  }

  getTotalSize(props) {
      props = props || this.props

      var domNode = findDOMNode(this)

      return domNode[getClientSizeName(props)]
  }

  adjustBefore(props, state, splitterSize) {
      var fixedIndex = getFixedIndex(props)
      var currentSize
      var currentSplit

      if (!~fixedIndex){
          if (!isPercentage(props.splitAt)){
              props.splitAt = this.getPercentageFor(parseFloat(props.splitAt) + splitterSize/2) + '%'
          }
          return
      }

      if (fixedIndex === 0){
          if (!isNumber(props.splitAt)){
              currentSize  = this.getTotalSize(props)
              currentSplit = parseFloat(props.splitAt)
              //split as is not a number, but a percentage, but we have to make it a number
              props.splitAt = currentSplit * currentSize / 100 - splitterSize / 2
          }
      }
  }

  getPercentageFor(size, props){

      var totalSize  = this.getTotalSize(props)

      return size * 100 / totalSize
  }

  adjustAfter(props, state, childrenStyles){
      var collapsedInfo = this.getCollapsedInfo(props, state)

      if (collapsedInfo.first || collapsedInfo.second){
          return
      }

      if (getFixedIndex(props) === 1){
          var sizeName       = getSizeName(props)
          var clientSizeName = getClientSizeName(props)
          var domNode        = findDOMNode(this).children[2]
          var size           = domNode[clientSizeName]

          childrenStyles.second[sizeName] = size
          childrenStyles.first[sizeName]  = calc('100%', -props.splitterStyle[sizeName], -size)
      }

  }

  shouldAdjust(props) {
      if (this.state.dragging){
          return false
      }

      var fixedIndex = getFixedIndex(props)

      if (!~fixedIndex){
          //when there is no fixed index, both have to be flexible,
          //so splitAt must be a percentage value.
          //if it's not, we should adjust
          return !isPercentage(props.splitAt)
      }
      if (fixedIndex === 0){
          //when the first should be fixed,
          //we only have to adjust if splitAt is not a number, but a percentage
          return !isNumber(props.splitAt)
      }

      //fixed index is 1
      //so we always have to adjust
      return true
  }

  handleSplitterMouseDown(event){
    this.props.onSplitterMouseDown(event)
  }

  handleSplitterDragStart(diff, event){
    this.props[this.props.onDragStartName](diff, event)
  }

  getCurrentSplitAt(){
    const clientSizeName = getClientSizeName(this.state.props)
    const firstChild = findDOMNode(this).children[0]
    const splitAt = firstChild[clientSizeName]

    return splitAt
  }

  handleSplitterDrag(diff, event) {
    const positions = getPositionNames(this.props)
    const posName = positions[0]
    let pos = diff[posName]

    this.props[this.props.onDragName](diff, event)

    pos = diff[posName]

    const dragSplitterStyle = {}

    dragSplitterStyle[positions[0]] = pos

    this.setState({
        dragging         : true,
        dragOffset       : pos,
        dragSplitterStyle: dragSplitterStyle
    })

    return pos
  }

  handleSplitterDrop(diff, event){

      var positions = getPositionNames(this.props)
      var posName   = positions[0]
      var pos       = diff[posName]

      var state = {
          dragging         : false,
          dragOffset       : 0,
          dragSplitterStyle: null
      }

      if (pos){
          var props          = this.state.props
          var currentSplitAt = this.getCurrentSplitAt()

          var propsForAdjust = {
              splitAt    : currentSplitAt,
              orientation: this.props.orientation,
              fixedIndex : this.props.fixedIndex
          }

          this.adjustBefore(propsForAdjust, this.state, props.splitterSize)

          var splitAt = propsForAdjust.splitAt

          if (this.props.stateful){
              state.splitAt = splitAt
          }
      }

      this.setState(state)

      if (!pos){
          return
      }

      //for the onDrop handler, we need to make sure we return the splitAt
      //in the exact same format as it was passed in the original props (either number or percentage)
      if (this.asPercentage != isPercentage(splitAt)){

          if (this.asPercentage && !isPercentage(splitAt)){
              splitAt = this.getPercentageFor(splitAt + props.splitterSize/2)
          }

          if (!this.asPercentage && isPercentage(splitAt)){
              splitAt = currentSplitAt
          }

          if (this.asPercentage && !isPercentage(splitAt)){
              splitAt += '%'
          }
      }

      this.props[this.props.onDropName](splitAt, event)
  }

  handleCollapse(direction){
      var state = {}

      if (direction == -1){
          state.firstCollapsed = true
      } else {
          state.secondCollapsed = true
      }

      if (this.props.stateful){
          this.setState(state)
      }

      this.props[this.props.onCollapseName](state.firstCollapsed? 0: 1)
  }

  handleExpand(direction){
      var state = {}

      if (direction == -1){
          state.firstCollapsed = false
      } else {
          state.secondCollapsed = false
      }

      if (this.props.stateful){
          this.setState(state)
      }

      this.props[this.props.onExpandName](state.firstCollapsed === false? 0: 1)
  }
}

SplitContainer.propTypes = {
  children: function(props, propName){
    var propValue = React.Children.toArray(props[propName])

    var isArray = Array.isArray(propValue)
    var isTwo   = isArray && propValue.length == 2

    if (!propValue || !isArray || !isTwo){
        console.warn('Please pass only 2 children. You provided ' + (isArray? propValue.length + ' children': propValue))
    }
  },

  orientation: PropTypes.oneOf(['horizontal','vertical']),

  splitterSize: PropTypes.number,
  locked      : PropTypes.bool,
  stateful    : PropTypes.bool
}

SplitContainer.defaultProps = assign({
  theme: 'default',
  orientation: 'horizontal',
  defaultStyle: {
  },
  defaultHorizontalStyle:{
  },
  defaultVerticalStyle: {
  },
  defaultSplitterStyle: {
  },
  defaultHorizontalSplitterStyle: {
  },
  defaultVerticalSplitterStyle: {
  },

  // defaultSplitterSize: 12,
  arrowSize: 6,
  arrowColor: 'rgb(118, 118, 118)',
  arrowPadding: 10,
  splitAt: '50%',

  fixedIndex         : null,
  locked             : false,
  stateful           : false,
  onDrop             : emptyFn,
  onDragStart        : emptyFn,
  onDrag             : emptyFn,
  onSplitterMouseDown: emptyFn,
  onExpand           : emptyFn,
  onCollapse         : emptyFn,

  onDragStartName: 'onDragStart',
  onDragName     : 'onDrag',
  onDropName     : 'onDrop',
  onExpandName   : 'onExpand',
  onCollapseName : 'onCollapse'

}, STYLES.defaults)
